#*****************************************************************************
#
#*****************************************************************************

package Mail::Maildir;

require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(
);

$VERSION = "0.2.3";

sub new {

  my $classname = shift;
  my $self = {
  	FCOUNT => 0,
  };

  bless($self,$classname);
  $self->_init( @_ );

  return $self;
}


sub _init {
  my $self = shift;

  if ( @_ && (scalar( @_ ) % 2 == 0) )
    {
      my %hashargs = @_;
      if ( defined($hashargs{DIR} ) )
	{
	  foreach my $key ( keys %hashargs )
	    {
	      $self->{$key} = $hashargs{$key};
	    }
	}
      else {$self->_initOldStyle( @_ );}
    }
  else {$self->_initOldStyle( @_ );}

  $self->{HOSTNAME} = `hostname`;		# XXX Fix this
  chomp $self->{HOSTNAME};
}

#******************************************************************************
#* initialize using the old positional parameter style new - deprecated
#******************************************************************************
sub _initOldStyle {
  my $self = shift;
  $self->Dir( shift );
  $self->PlusPlus( shift );
  $self->CourierHack( shift );
}

sub Create {

	my $me = shift;

	my($path) = $me->{DIR};

	mkdir($path);
	mkdir($path . "/new");
	mkdir($path . "/cur");
	mkdir($path . "/tmp");

	if ( $me->{COURIERHACK} ) {
		symlink(".", $path . "/Maildir");
	}
}

sub Open {
	my $me = shift; 
	$me->Rescan;
}

sub Rescan {

	my $me = shift; 

	my $path  = $me->{DIR};

	my (@allfiles);

	opendir(DIR,"$path/new");

	@allfiles = grep !/^\./, readdir DIR;

	map s/^/new\//g, @allfiles;

	closedir(DIR);

	opendir(DIR,"$path/cur");

	my @tmpfiles = grep !/^\./, readdir DIR;

	map s/^/cur\//g, @tmpfiles;

	push (@allfiles,@tmpfiles);

	closedir(DIR);

	$me->{FILES} = \@allfiles;

	$me->{COUNT} = $#allfiles + 1;

	if ( $me->{PLUSPLUS} ) {
		opendir(DIR,"$path");
		my @folders = grep ! /^\.(\.)?$/, (grep /^\./, readdir DIR);
		map s/^.// , @folders;
		closedir(DIR);

		$me->{FOLDERS} = \@folders;
	}

}

sub PutFile {
	my $me = shift; 

	my($path) = shift;

	my($filename) = $me->filename;

	$tmpfile = $me->{DIR} . "/" . $me->{CWF} . "/tmp/$filename";
	$newfile = $me->{DIR} . "/" . $me->{CWF} . "/new/$filename";

	open(FILE,"$path") || die "Can't open $path";
	open(MFILE,">$tmpfile") || 
		die "Can't open $tmpfile";

	while(<FILE>) {
		print MFILE;
	}

	close(MFILE);
	close(FILE);

	rename "$tmpfile","$newfile" || die;

	push(@{$me->{FILES}}, "new/$filename");

	$me->{COUNT}++;

}

sub Delete {

	my $me = shift; 

	my $num = shift;

	$filename = $me->{DIR} . "/" . $me->GetFile($num);

	unlink $filename;

}

sub filename {
  	my $me = shift;

	$filename = time . "." . $$ . "_" . $me->{FCOUNT} . "." 
			. $me->{HOSTNAME};
	chomp $filename;
	$me->{FCOUNT}++;
	return $filename;
}

sub Dir {
  my $me = shift;
  my $dir = shift or return $me->{DIR};
  $me->{DIR} = $dir;
}

sub PlusPlus {
  my $me = shift;
  my $plusplus = shift or return $me->{PLUSPLUS};
  $me->{PLUSPLUS} = $plusplus;
}

sub CourierHack {
  my $me = shift;
  my $courierhack = shift or return $me->{COURIERHACK};
  $me->{COURIERHACK} = $courierhack;
}

sub GetFile {
	my $me = shift; 

	my($num) = shift;

	return ${$me->{FILES}}[$num];
}

sub CreateFolder {
	my $me = shift; 

	return 0 if (! $me->{PLUSPLUS} );

	my($folder) = shift;

	return 0 if ( $folder eq "INBOX" );

	my($md) = $me->{DIR} . "/" . "." . "$folder";
	mkdir($md) || return 0;

	my($mf) = $md . "/maildirfolder";

	if ( ! -f $mf ) {
		open(MF,">>$mf");
		close(MF);
	}

	mkdir($md . "/tmp");
	mkdir($md . "/cur");
	mkdir($md . "/new");

	if ($me->{COURIERHACK}) {
		open(IMAP,">>" . $me->{DIR} . "/courierimapsubscribed");
		print IMAP  "INBOX." . $folder . "\n";
		close(IMAP);
	}

	push(@{$me->{FOLDERS}}, "$folder");

	return 1;
}

sub Folders {
	my $me = shift;

	return @{$me->{FOLDERS}};
}

sub Folder {
  my $me = shift;
  my $folder = shift or return substr($me->{CWF},1);

  if ($folder eq "INBOX") { 
  	$me->{CWF} = "";
  } else {
  	$me->{CWF} = "." . $folder;
  }
}

# Deprecated 
sub ChFolder {
	my $me = shift; 
	$me->Folder;
}


1;
__END__

##############################################################################
# POD Documentation (perldoc Mail::Maildir or pod2html this_file)
##############################################################################

=head1 NAME

Mail::Maildir - Perl 5 module to read and write Maildir mailboxes

=head1 SYNOPSIS

  use Mail::Maildir;

  $maildir = new Mail::Maildir(DIR => "/tmp/maildir", PLUSPLUS=> 0);

  $maildir->Create;

  $maildir->Open;

  $maildir->PutFile("/tmp/blah");

  $maildir->Delete(1);

  print $maildir->GetFile(2);

=head1 DESCRIPTION

This module implements an interface to Maildir and Maildir++ mail directories.

=head1 AUTHOR

Paul Dwerryhouse <paul@dwerryhouse.com.au>

=head1 CREDITS

Based around Mail::POP3Client by Sean Dowd <pop3client@dowds.net>

=head1 SEE ALSO

perl(1).

=cut
